<script type="text/x-template" id="tmpl-sensors">
    <div class="sensors">
        <div class="head">
            <button type="button" title="Refresh measurements" @click="refresh">
                <i class="fa fa-sync"></i>
            </button>
        </div>

        <div class="body">
            <sensor-metric v-for="value, name in metrics"
                           :key="name"
                           :name="name[0].toUpperCase() + name.slice(1)"
                           :value="value">
            </sensor-metric>
        </div>
    </div>
</script>

<script type="text/x-template" id="tmpl-sensor-metric">
    <div class="metric">
        <div class="col-6 name" v-text="name"></div>
        <div class="col-6 value" v-text="value"></div>
    </div>
</script>
