.sensors .head{text-align:right}.sensors .head button{border:0}.sensors .head button:hover{background:#def6ea}.sensors .body{display:flex;flex-direction:column;margin:2rem auto;width:80%}.sensors .body .metric{margin-bottom:.75rem;padding:2rem;border:1px solid #ccc;border-radius:2rem}.sensors .body .metric:nth-child(odd){background:rgba(255,255,255,0)}.sensors .body .metric:nth-child(even){background:#f1f3f2}.sensors .body .metric:hover{background:#def6ea}.sensors .body .metric .value{text-align:right;font-weight:bold}
