@import 'common/vars';

.widget .date-time-weather {
    height: 100%;
    display: flex;
    flex-direction: column;
    align-items: center;
    padding-top: 1rem;

    .date {
        font-size: 1.3em;
        height: 10%;
    }

    .time {
        font-size: 2em;
        height: 14%;
    }

    .weather {
        height: 35%;
        display: flex;
        align-items: center;

        .temperature {
            font-size: 2em;
            margin-left: 1rem;
        }
    }

    .summary {
        height: 28%;
    }

    .sensors {
        width: 100%;
        height: 13%;

        .sensor {
            padding: 0 1rem;
        }

        .humidity {
            text-align: right;
        }
    }
}

