.media-plugin {
    .devices {
        display: inline-block;

        button {
            padding: .5rem;
            margin-right: .5rem;

            &.selected {
                background: initial;
                color: $default-hover-fg;

                &:hover {
                    color: $default-hover-fg-2;
                }
            }
        }

        .dropdown {
            white-space: nowrap;

            .item {
                display: flex;
                align-items: center;

                .text {
                    text-align: left;
                    margin-left: 2rem;
                }

                &:first-child {
                    border-bottom: $default-border-3;
                    color: $devices-dropdown-refresh-fg;
                    font-size: .8em;
                }

                &:hover {
                    background: $hover-bg
                }
            }
        }
    }
}

