$button-enabled-color: #59df3e;
$button-hover-color: $button-enabled-color;
$play-button-hover-color: #64ef4a;

$control-panel-bg: rgba(245,245,245,0.95);
$control-panel-height: 10rem !default;
$control-panel-shadow: 0 -2.5px 4px 0 #c0c0c0;
$control-time-color: #666;

$empty-results-color: #506050;
$result-item-icon: #444;

$devices-dropdown-z-index: 2;
$devices-dropdown-refresh-fg: #666;

$subs-control-height: 4rem;
$btn-default-shadow: 2px 2px 2px #ddd;
$btn-hover-default-shadow: 3px 3px 3px #ddd;

$torrents-head-bg: #e8e8e8;

