.tts-container{max-width:80rem;min-height:10rem;margin:3rem auto;display:flex;align-items:center;justify-content:center;border:1px solid #ddd;border-radius:3rem}.tts-container form{margin:0;width:90%}.tts-container form input[type=text]{width:100%}.tts-container form button{border-radius:5rem}.tts-container form input:hover,.tts-container form button:hover{border-color:#35b870}
