.widget .rss-news{height:100%;display:flex;align-items:center}.widget .rss-news .article{width:90%;padding:0 2em}.widget .rss-news .article .source{font-size:1.7em;font-weight:bold;margin-bottom:.5em}.widget .rss-news .article .title{font-size:1.7em;margin-bottom:.5em}.widget .rss-news .article .published{text-align:right;font-size:1em}
