<script type="text/x-template" id="tmpl-switch">
    <div class="switch" :class="{glow: glow}" @click="toggled">
        <input type="checkbox" v-model="value">
        <label></label>
    </div>
</script>

<script type="application/javascript" src="{{ url_for('static', filename='js/elements/switch.js') }}"></script>

