button[disabled],
.button[disabled] {
    color: #bbb;
    background: rgba(240,240,240,1);
    border: 1px solid;
}

.btn-primary {
    background-color: #d8ffe0 !important;
    border: 1px solid #c2f0cf !important;
}

