.fa.fa-zwave:before{content:' ';background:url('/static/img/icons/z-wave-logo.png');background-size:1em 1em;width:1em;height:1em;display:inline-block}.zwave-container{height:100%;padding:0 .5em;background:#f1f1f1;display:flex;flex-direction:column;align-items:center;overflow:auto}.zwave-container .no-items{padding:2em;font-size:1.5em;color:#555;display:flex;align-items:center;justify-content:center}.zwave-container .view-options{display:flex;width:100%;justify-content:space-between;padding:1em 0}.zwave-container .view-options .view-selector{display:inline-flex}.zwave-container .view-options .buttons{display:inline-flex}.zwave-container .view-options select{width:100%;border-radius:1em}.zwave-container .btn-default{border:0;padding:0 1em}.zwave-container .btn-default:hover{border:1px solid #ddd;border-radius:1em}.zwave-container .buttons{text-align:right}.zwave-container .view{min-width:400pt;max-width:750pt;background:#fff;border:1px solid #d8d8d8;border-radius:1.5em;box-shadow:1px 2px 2px #ccc}.zwave-container .item.selected{box-shadow:0 2px 4px 0 #bbb}.zwave-container .item .name{padding:1em;cursor:pointer;text-transform:uppercase;letter-spacing:.06em}.zwave-container .item .name.selected{border-radius:1.5em}.zwave-container .item:hover{background:#def6ea}.zwave-container .item:not(:last-child){border-bottom:1px solid #ddd}.zwave-container .item:first-child{border-radius:1.5em 1.5em 0 0}.zwave-container .item:last-child{border-radius:0 0 1.5em 1.5em}.zwave-container .params{background:#fff;padding-bottom:1em}.zwave-container .params .section{display:flex;flex-direction:column;padding:0 1em}.zwave-container .params .section:not(:first-child){padding-top:1em}.zwave-container .params .section .header{display:flex;align-items:center;font-weight:bold;border-bottom:1px solid #e8e8e8}.zwave-container .params .row{display:flex;align-items:center;border-radius:1em;padding:.3em}.zwave-container .params .row:nth-child(even){background:#ededed}.zwave-container .params .row:nth-child(odd){background:#fff}.zwave-container .params .row:hover{background:#def6ea}.zwave-container .params .param-name{display:inline-flex;width:40%;margin-left:1%;vertical-align:top;letter-spacing:.04em}.zwave-container .params .param-value{display:inline-block;width:58%;text-align:right}.zwave-container .params .param-value .value-edit{display:flex;align-items:center}.zwave-container .params .param-value .value-data{display:inline-block;font-weight:bold}.zwave-container .params .param-value .slider-container{display:flex;align-items:center}.zwave-container .params .param-value .unit{font-size:.8em;margin-left:1em;display:inline}.zwave-container .params .param-value select{width:100%;border-radius:2em}.zwave-container .params .param-value .numeric input.slider{text-align:left}.zwave-container .params .param-value .numeric input[type=text]{text-align:right;width:100%}.zwave-container .params .param-value .numeric .row{background:none}.zwave-container .params .param-value .numeric .row:hover{background:none}.zwave-container .params .param-value .numeric .value-min,.zwave-container .params .param-value .numeric .value-max{width:50%;font-size:.85em;opacity:.75}.zwave-container .params .param-value .numeric .value-min{text-align:left}.zwave-container .params .param-value .numeric .value-max{text-align:right}.zwave-container .btn-value-name-edit{padding:0}.zwave-container .modal .section .header{background:none;padding:.5em 0}.zwave-container .modal .section .body{padding:0}.zwave-container .modal .network-info{min-width:600pt}.zwave-container .error{color:#a00}.zwave-container .node .actions .row,.zwave-container .scene .actions .row{cursor:pointer}.zwave-container .node form,.zwave-container .scene form{margin-bottom:0}
