@import 'common/vars';
@import 'common/mixins';
@import 'common/layout';
@import 'common/animations';

@import 'webpanel/plugins/media/vars';
@import 'webpanel/plugins/media/search';
@import 'webpanel/plugins/media/devices';
@import 'webpanel/plugins/media/results';
@import 'webpanel/plugins/media/controls';
@import 'webpanel/plugins/media/info';
@import 'webpanel/plugins/media/subs';
@import 'webpanel/plugins/media/torrents';

.media-plugin {
    display: flex;
    flex-direction: column;
    height: inherit;
    letter-spacing: .03rem;

    .dropdown {
        z-index: $devices-dropdown-z-index;

        .item {
            display: flex;
            align-items: center;

            .text {
                margin-left: 1rem;
            }

            &:hover {
                background: $hover-bg;
            }
        }
    }

    input[type=text] {
        width: 100%;

        &:hover {
            border-color: $default-hover-fg;
        }
    }

    button {
        border: 0;
    }

    .icon {
        color: $result-item-icon;
        margin-right: .5em;
    }

    .top-buttons {
        text-align: right;
        float: right;
    }
}

