.widget .date-time-weather{height:100%;display:flex;flex-direction:column;align-items:center;padding-top:1rem}.widget .date-time-weather .date{font-size:1.3em;height:10%}.widget .date-time-weather .time{font-size:2em;height:14%}.widget .date-time-weather .weather{height:35%;display:flex;align-items:center}.widget .date-time-weather .weather .temperature{font-size:2em;margin-left:1rem}.widget .date-time-weather .summary{height:28%}.widget .date-time-weather .sensors{width:100%;height:13%}.widget .date-time-weather .sensors .sensor{padding:0 1rem}.widget .date-time-weather .sensors .humidity{text-align:right}
