.media-plugin {
    #media-info {
        .modal {
            max-width: 90%;
        }
    }

    .info-container {
        .row {
            display: flex;
            align-items: center;
            padding: .75em .5em;
            border-bottom: $default-border-2;

            &:nth-child(odd) { background: rgba(255,255,255,0.0); }
            &:nth-child(even) { background: $default-bg-3; }

            &:hover {
                background: $hover-bg;
                border-radius: 1em;
            }

            .attr {
                font-size: 1.1em;
            }

            .value {
                text-align: right;
            }
        }
    }
}

