@import 'common/vars';

.widget .rss-news {
    height: 100%;
    display: flex;
    align-items: center;

    .article {
        width: 90%;
        padding: 0 2em;

        .source {
            font-size: 1.7em;
            font-weight: bold;
            margin-bottom: .5em;
        }

        .title {
            font-size: 1.7em;
            margin-bottom: .5em;
        }

        .published {
            text-align: right;
            font-size: 1em;
        }
    }
}

