$icon-color: #7e8;
$icon-border: 1px solid #ccc;
$icon-shadow: 2px 2px 2px #ccc;

#assistant-google-modal {
    .modal {
        width: 50vw;
        height: 50vh;

        .body {
            width: 100%;
            height: 100%;
            display: flex;
            align-items: center;
            justify-content: center;
            flex-direction: column;
            text-align: center;

            .icon {
                font-size: 3em;
                color: $icon-color;
                box-shadow: $icon-shadow;
                border: $icon-border;
                border-radius: 3em;
                padding: .7em 1em .5em 1em;
            }

            .text {
                margin-top: 2.5em;
            }
        }
    }
}

