$button-enabled-color: #59df3e;
$button-hover-color: $button-enabled-color;
$play-button-hover-color: #64ef4a;

$duration-color: #666;

$control-panel-height: 10rem !default;
$control-panel-bg: rgba(245,245,245,0.95);
$control-panel-shadow: 0 -2.5px 4px 0 #c0c0c0;
$control-time-color: #666;

$browser-panel-bg: rgba(248,250,250,0.95);

$empty-playlist-color: rgba(200,200,200,0.7);
$empty-playlist-shadow: 2px 1px rgb(235,235,235);
$playlist-controls-bg: rgba(247,247,247,0.95);
$playlist-controls-border: $default-border-2;

$move-mode-track-border: 3px dotted rgb(216,156,136);
$move-mode-track-bg: rgba(216,156,136,0.3);

$search-modal-footer-border: 1px solid #ccc;

$info-modal-row-border: 1px solid #ddd;
$info-modal-attr-color: #777;
$track-info-hover-color: rgb(46,190,110);
$filter-bar-shadow: 0 2.5px 4px 0 #bbb;

