<link rel="stylesheet" type="text/css" href="//fonts.googleapis.com/css?family=Lato">
<link rel="stylesheet" type="text/css" href="{{ url_for('static', filename='css/skeleton.css') }}">
<link rel="stylesheet" type="text/css" href="{{ url_for('static', filename='css/normalize.css') }}">
<link rel="stylesheet" href="https://use.fontawesome.com/releases/v5.12.0/css/all.css"
      integrity="sha384-REHJTs1r2ErKBuJB0fCK99gCYsVjwxHrSU0N7I1zl9vZbggVJXRMsv/sLlOAGb4M" crossorigin="anonymous">
