<div class="row plugin tts-container">
    <form @submit="talk">
        <div class="col-8">
            <input type="text" name="text" placeholder="Text to say">
        </div>
        <div class="col-2">
            <input type="text" name="language" placeholder="Language code">
        </div>
        <div class="col-1">
            <button type="submit">
                <i class="fa fa-volume-up"></i>
            </button>
        </div>
    </form>
</div>
