<script type="application/javascript" src="{{ url_for('static', filename='js/plugins/media/item.js') }}"></script>

<script type="text/x-template" id="tmpl-media-item">
    <div class="media-item"
         :class="{selected: selected, active: active}"
         @click="onClick">
        <i class="icon" :class="item.handler.iconClass" v-if="item.handler">&nbsp; </i>
        <span v-text="item.title"></span>
    </div>
</script>

