@import 'common/vars';

@import 'common/mixins';
@import 'common/layout';
@import 'common/elements';

body {
    width: 100vw;
    height: 100vh;
    margin: 0;
    overflow-x: hidden;
    font-family: $default-font-family;
    font-size: $default-font-size;
}

main {
    height: 100%;
    display: flex;
    align-items: center;
    justify-content: center;
    margin: 0;

    form {
        border: $default-border-3;
        border-radius: 3em;
        padding: 4em;

        .row {
            margin: 1em 0;
        }

        input[type=text],
        input[type=password] {
            width: 100%;
        }

        input[type=submit] {
            border-radius: 1em;
        }
    }
}

a {
    color: $default-link-fg;
}

