<!doctype html>
<head>
    <title>Platypush Settings</title>

    <meta http-equiv="X-UA-Compatible" content="IE=edge">

    {% include 'css-common.html' %}
    <link rel="stylesheet" href="{{ url_for('static', filename='css/dist/settings.css') }}">

    <script type="text/javascript">
        if (!window.config) {
            window.config = {};
        }

        {% if token %}
            window.config.token = '{{ token }}';
        {% else %}
            window.config.token = undefined;
        {% endif %}
    </script>

    {% include 'js-common.html' %}
    {% include 'elements.html' %}
</head>

<body>
<div id="app">
    {% include 'settings/nav.html' %}

    <main class="col-s-8 col-m-9 col-l-10">
        {% include 'settings/sections/users.html' %}
        {% include 'settings/sections/token.html' %}
    </main>

    {% include 'notifications.html' %}
</div>

<script type="text/javascript" src="{{ url_for('static', filename='js/settings/index.js') }}"></script>
</body>

