@import 'common/vars';

.widget .image-carousel {
    height: 100%;
    position: relative;
    display: flex;
    align-items: center;
    justify-content: center;
    background-color: rgba(0,0,0,0);

    .background {
        position: absolute;
        top: 0;
        width: 100%;
        height: 100vh;
        background-color: rgba(0,0,0,0);
        background-position: center;
        background-size: cover;
        background-repeat: no-repeat;
        filter: blur(13px);
        -webkit-filter: blur(13px);
    }

    img {
        position: absolute;
        max-height: 100%;
        z-index: 2;
    }
}

