<script type="text/x-template" id="tmpl-light-hue-scene">
    <div class="scene" @click="clicked">
        <div v-text="name"></div>
    </div>
</script>

<script type="application/javascript" src="{{ url_for('static', filename='js/plugins/light.hue/scenes.js') }}"></script>

