<script type="text/x-template" id="tmpl-dropdown">
    <div class="dropdown" :id="id" :class="{hidden: !visible}">
        <div class="row item"
             :class="{disabled: item.disabled, ...classes.reduce((classes, c) => {classes[c] = true; return classes}, {})}"
             v-for="item in items" @click="clicked(item)">
            <div class="col-1 icon">
                <i class="fa" :class="['fa-' + (item.icon || '')]" v-if="item.icon"></i>
                <i :class="item.iconClass" v-else-if="item.iconClass"></i>
                <img src="item.img" v-else-if="item.image">
            </div>
            <div class="col-11 text" v-text="item.text"></div>
        </div>
    </div>
</script>

<script type="application/javascript" src="{{ url_for('static', filename='js/elements/dropdown.js') }}"></script>

