@import 'common/vars';

.input-icon {
    position: absolute;
    min-width: 3rem;
    padding: 1rem;
    color: $text-icon-color;
}

input[type=text],
input[type=password] {
    border-radius: 5rem;

    &:hover {
        border: $border-hover;
    }

    &:focus {
        border: $border-focus;
    }

    &.with-icon {
        padding-left: 3rem;
    }
}

