.gpio-container{display:flex;flex-direction:column;height:100%;overflow:auto}.gpio-container .refresh{border-bottom:1px solid #ddd;background:#f8f8f8}.gpio-container .refresh button{border:0}.gpio-container .refresh button:hover{color:#35b870}.gpio-container .pins .pin{padding:1em;border-bottom:1px solid #ddd;display:flex;flex-direction:row;align-items:center}.gpio-container .pins .pin:nth-child(even){background:#f0f0f0}.gpio-container .pins .pin:hover{background:#def6ea}
