<script type="application/javascript" src="{{ url_for('static', filename='js/plugins/music.mpd/playlist.js') }}"></script>

<script type="text/x-template" id="tmpl-music-mpd-playlist-item">
    <div class="row item playlist-item"
            :class="{selected: selected, active: active, move: move}"
            @click="$emit('input', track)">
        <div class="col-5 artist" v-text="track.artist" v-if="track.artist"></div>
        <div class="col-5 artist empty" v-else>[No Artist]</div>

        <div class="col-5 title" v-text="track.title || track.file"></div>
        <div class="col-2 pull-right duration" v-text="convertTime(track.time)"></div>
    </div>
</script>

