@import 'common/vars';

.camera {
    min-height: 90%;
    margin-top: 4%;
    overflow: auto;
    display: flex;
    flex-direction: column;
    align-items: center;

    .camera-container {
        min-width: 640px;
        min-height: 480px;
        position: relative;
        background: black;
        margin-bottom: 1em;

        .frame, .no-frame {
            position: absolute;
            top: 0;
            width: 100%;
            height: 100%;
        }

        .frame {
            z-index: 1;
        }

        .no-frame {
            display: flex;
            background: rgba(0, 0, 0, 0.1);
            color: white;
            align-items: center;
            justify-content: center;
            z-index: 2;
        }
    }
}

