#assistant-google-modal .modal{width:50vw;height:50vh}#assistant-google-modal .modal .body{width:100%;height:100%;display:flex;align-items:center;justify-content:center;flex-direction:column;text-align:center}#assistant-google-modal .modal .body .icon{font-size:3em;color:#7e8;box-shadow:2px 2px 2px #ccc;border:1px solid #ccc;border-radius:3em;padding:.7em 1em .5em 1em}#assistant-google-modal .modal .body .text{margin-top:2.5em}
