<script type="text/x-template" id="tmpl-widget-image-carousel">
    <div class="image-carousel">
        <div ref="background" class="background"></div>
        <img ref="img" :src="currentImage" alt="Your carousel images">
    </div>
</script>
