.switches-root .switch-root .head{padding:1rem .5rem;background:#e8e8e8;border-top:1px solid #ddd;border-bottom:1px solid #ddd;text-transform:uppercase}.switches-root .switch-root .switches{display:flex;flex-direction:column;padding:1rem}.switches-root .switch-root .device{display:flex;align-items:center}.switches-root .switch-root .device .toggle{text-align:right}.switches-root .switch-root .device:hover{background:#def6ea}.switches-root .switch-root .device:not(:last-child){border-bottom:1px solid #ddd}
