//// General purpose classes and rules /////
.hidden {
    display: none !important;
}

.selected {
    background: $selected-bg;
}

.pull-right {
    text-align: right !important;
}

.clickable {
    cursor: pointer;
}

a:focus {
    outline: none;
}

::-moz-focus-outer,
::-moz-focus-inner {
    border: 0;
}

select:-moz-focusring {
    color: transparent;
    text-shadow: 0 0 0 #000;
}

//// UI elements definitions /////

@import 'common/elements/button';
@import 'common/elements/switch';
@import 'common/elements/range-slider';
@import 'common/elements/slider';
@import 'common/elements/text';
@import 'common/elements/dropdown';
@import 'common/elements/autocomplete';

