@import 'common/vars';

.widget .calendar {
    padding: 1rem;

    .upcoming-event {
        text-align: center;
        margin-bottom: 1.5rem;

        .summary {
            text-transform: uppercase;
            font-size: 1.35em;
        }
    }
}

