.widget .music{height:100%;display:flex;flex-direction:column;align-items:center;justify-content:center;position:relative}.widget .music .track{text-align:center}.widget .music .track .unknown,.widget .music .track .no-track{font-size:2em}.widget .music .track .artist{font-size:1.9em;font-weight:bold;margin-bottom:.25em}.widget .music .track .title{font-size:1.8em}.widget .music .time{width:100%;margin-top:1em;font-size:1.2em}.widget .music .time .row{padding:0 .5em}.widget .music .time .time-total{text-align:right}.widget .music .time .progress-bar{width:100%;height:1em;position:relative;margin-bottom:.75em}.widget .music .time .progress-bar .total{position:absolute;width:100%;height:100%;top:0;background:#ddd;border-radius:5rem}.widget .music .time .progress-bar .elapsed{position:absolute;width:100%;height:100%;top:0;background:#c8ffd0;border-radius:5rem;z-index:1}.widget .music .playback-status{position:absolute;bottom:0;border-top:1px solid #ddd;color:#757f70;width:100%;height:2em}.widget .music .playback-status .status-property{display:flex;align-items:center;justify-content:center;height:100%}.widget .music .playback-status .active{color:#35b870}
